/* DSP LABORATORY 
 * University of Technology Graz, 2018
 * Josef Kulmer
 * 
 * UNIT 1, part 1: No output
 * 
 * 
 */
#include <iostream>
#include <math.h>
#include "portaudio.h"
#include "blockprocessing.h"

struct UserDataExperiment{
    int checkCallback;
};

// the callback function, evaluated at each interrupt
static int paCallback(const void *input, void *output,
    unsigned long framelength,
    const PaStreamCallbackTimeInfo* timeInfo,
    PaStreamCallbackFlags statusFlags,
    void *userdata)
{
    float *in = (float*)input;
    float *out = (float*)output;
    
    UserDataExperiment *ud = (UserDataExperiment*)userdata;
    
    for(int i=0;i<framelength;i++){
        *out++ = 0; // Zero output
    }
    
    ud->checkCallback ++; // report that callback function is working

    return 0;
}

int main(int argc, char *argv[]) {   
    PaStream *stream;
    PaError err;
    int checkCallback_local = 0; // check whether callback function is still active
    int iTimecounter = 0;    
    UserDataExperiment ud; // create objects
    ud.checkCallback = 0; // a watchdog if callback is still working (necessary for RPi) 
    
    // the following loop stops and restarts the portaudio stream if it is broken
    do{
        err = Pa_Initialize();
        std::cout << "Initialize portaudio: " << Pa_GetErrorText( err ) << std::endl;

        err = Pa_OpenDefaultStream(&stream, 
            NINPUTCHANNELS, 
            NOUTPUTCHANNELS, 
            paFloat32, 
            FS, 
            FRAMELENGTH, 
            paCallback, 
            &ud);
        std::cout << "Open audio stream: " << Pa_GetErrorText( err ) << std::endl;

        err = Pa_StartStream(stream);
        std::cout << "Start audio stream: " << Pa_GetErrorText( err ) << std::endl;

        Pa_Sleep(TIMETOSLEEP); // wait until Stream is open
        
        // wait in this loop until DURATION is reached or callback function needs to be restarted
        while(iTimecounter < DURATION-1 && ud.checkCallback != checkCallback_local){ // there was a callback within last second
            checkCallback_local = ud.checkCallback;
            Pa_Sleep(TIMETOSLEEP); // wait a second
            iTimecounter++;
        }
        
        if(iTimecounter != DURATION-1 && ud.checkCallback != checkCallback_local){
            std::cout << "There is a problem with portaudio -> restart" << std::endl;
        }

        err = Pa_StopStream(stream);
        std::cout << "Stop audio stream: " << Pa_GetErrorText( err ) << std::endl;

        err = Pa_CloseStream(stream);
        std::cout << "Close audio stream: " << Pa_GetErrorText( err ) << std::endl;
    }while(iTimecounter < DURATION-1);

    Pa_Terminate();
    return 0;    
}
